@@ -28,6 +28,10 @@ void gdm_info   (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 void gdm_error  (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 void gdm_debug  (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
+void gdm_fdprintf  (int fd, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
+int gdm_fdgetc     (int fd);
+char *gdm_fdgets   (int fd);
+
 /* Note that these can actually clear environment without killing
  * the LD_* env vars if --preserve-ld-vars was passed to the
  * main daemon */
@@ -53,6 +57,8 @@ void	gdm_sigchld_block_push (void);
 void	gdm_sigchld_block_pop (void);
 void	gdm_sigterm_block_push (void);
 void	gdm_sigterm_block_pop (void);
+void	gdm_sigusr2_block_push (void);
+void	gdm_sigusr2_block_pop (void);
 
 pid_t	gdm_fork_extra (void);
 void	gdm_wait_for_extra (int *status);
@@ -63,6 +69,25 @@ gboolean gdm_is_loopback_addr (struct in_addr *ia);
 
 gboolean gdm_setup_gids (const char *login, gid_t gid);
 
+void gdm_desetuid (void);
+
+gboolean gdm_test_opt (const char *cmd, const char *help, const char *option);
+
+void gdm_close_all_descriptors (int from, int except);
+
+gboolean gdm_is_a_no_password_user (const char *user);
+
+int gdm_open_dev_null (mode_t mode);
+
+void gdm_unset_signals (void);
+
+char * gdm_locale_to_utf8 (const char *text);
+char * gdm_locale_from_utf8 (const char *text);
+
+void gdm_saveenv (void);
+/* leaks */
+void gdm_restoreenv (void);
+
 #endif /* GDM_MISC_H */
 
 /* EOF */
