@@ -21,6 +21,9 @@
 
 #include "gdm.h"
 
+/* Wipe cookie files */
+void		gdm_server_wipe_cookies	(GdmDisplay *disp);
+
 gboolean	gdm_server_start	(GdmDisplay *d,
 					 gboolean treat_as_flexi,
 					 int min_flexi_disp,
@@ -32,6 +35,11 @@ GdmDisplay *	gdm_server_alloc	(gint id,
 void		gdm_server_whack_clients (GdmDisplay *disp);
 void		gdm_server_checklog	(GdmDisplay *disp);
 
+char **		gdm_server_resolve_command_line (GdmDisplay *disp,
+						 gboolean resolve_handled,
+						 const char *vtarg);
+
+
 
 #endif /* GDM_SERVER_H */
 
