@@ -17,10 +17,14 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <gtk/gtkmessagedialog.h>
 #include <syslog.h>
 #include <pwd.h>
 #include <shadow.h>
+#include <grp.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #ifdef HAVE_CRYPT
 #  include <crypt.h>
@@ -32,6 +36,7 @@
 #include "misc.h"
 #include "slave.h"
 #include "verify.h"
+#include "errorgui.h"
 
 /* Configuration option variables */
 extern gboolean GdmAllowRoot;
@@ -65,12 +70,13 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
 	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
 	    if (login == NULL ||
-		gdm_slave_greeter_check_interruption (login)) {
+		gdm_slave_greeter_check_interruption ()) {
 		    if (local)
 			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		    g_free (login);
 		    return NULL;
 	    }
+	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
     } else {
 	    login = g_strdup (username);
     }
@@ -91,14 +97,15 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 
     /* Request the user's password */
     if (pwent != NULL &&
-        ve_string_empty (ppasswd)) {
+        (ve_string_empty (ppasswd) ||
+	 (local && gdm_is_a_no_password_user (login)))) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
 	    passwd = gdm_slave_greeter_ctl (GDM_NOECHO, _("Password: "));
 	    if (passwd == NULL)
 		    passwd = g_strdup ("");
-	    if (gdm_slave_greeter_check_interruption (passwd)) {
+	    if (gdm_slave_greeter_check_interruption ()) {
 		    if (local)
 			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		    g_free (login);
@@ -116,12 +123,12 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    gdm_error (_("Couldn't authenticate user"));
 	    /* FIXME: Hmm, how are we sure that the login is username
 	     * and password.  That is the most common case but not
-	     * neccessairly true, this message needs to be changed
+	     * necessarily true, this message needs to be changed
 	     * to allow for such cases */
 	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password.  "
-		       "Letters must be typed in the correct case.  "  
-		       "Please be sure the Caps Lock key is not enabled"));
+		    (_("\nIncorrect username or password. "
+		       "Letters must be typed in the correct case. "  
+		       "Please make sure the Caps Lock key is not enabled."));
 	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
 	    g_free (auth_errmsg);
 	    g_free (login);
@@ -136,12 +143,12 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	    sleep (GdmRetryDelay);
 	    /* FIXME: Hmm, how are we sure that the login is username
 	     * and password.  That is the most common case but not
-	     * neccessairly true, this message needs to be changed
+	     * necessarily true, this message needs to be changed
 	     * to allow for such cases */
 	    auth_errmsg = g_strdup_printf
-		    (_("\nIncorrect username or password.  "
-		       "Letters must be typed in the correct case.  "  
-		       "Please be sure the Caps Lock key is not enabled"));
+		    (_("\nIncorrect username or password. "
+		       "Letters must be typed in the correct case. "  
+		       "Please make sure the Caps Lock key is not enabled."));
 	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
 	    g_free (auth_errmsg);
 	    g_free (login);
@@ -173,8 +180,8 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
 	 strcmp (pwent->pw_shell, "/bin/false") == 0)) {
 	    gdm_error (_("User %s not allowed to log in"), login);
 	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
-					  _("\nThe system administrator"
-					    " has disabled your "
+					  _("\nThe system administrator "
+					    "has disabled your "
 					    "account."));
 	    /*gdm_slave_greeter_ctl_no_ret (GDM_ERRDLG,
 	      _("Login disabled"));*/
@@ -186,6 +193,16 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
     g_free (passwd);
     g_free (ppasswd);
 
+    if ( ! gdm_setup_gids (login, pwent->pw_gid)) {
+	    gdm_error (_("Cannot set user group for %s"), login);
+	    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+					  _("\nCannot set your user group, "
+					    "you will not be able to log in, "
+					    "please contact your system administrator."));
+	    g_free (login);
+	    return NULL;
+    }
+
     return login;
 }
 
@@ -198,9 +215,28 @@ gdm_verify_user (GdmDisplay *d, const char *username, const gchar *display, gboo
  * session for this user
  */
 
-void
+gboolean
 gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display) 
 {
+	struct passwd *pwent;
+
+	pwent = getpwnam (login);
+	if (pwent == NULL) {
+		gdm_error (_("Cannot get passwd structure for %s"), login);
+		return FALSE;
+	}
+
+	if ( ! gdm_setup_gids (login, pwent->pw_gid)) {
+		gdm_error (_("Cannot set user group for %s"), login);
+		gdm_error_box (d,
+			       GTK_MESSAGE_ERROR,
+			       _("\nCannot set your user group, "
+				 "you will not be able to log in, "
+				 "please contact your system administrator."));
+		return FALSE;
+	}
+
+	return TRUE;
 }
 
 
@@ -212,6 +248,12 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display)
 void
 gdm_verify_cleanup (GdmDisplay *d)
 {
+	gid_t groups[1] = { 0 };
+
+	/* Clear the group setup */
+	setgid (0);
+	/* this will get rid of any suplementary groups etc... */
+	setgroups (1, groups);
 }
 
 
@@ -230,7 +272,7 @@ gdm_verify_check (void)
 
 /* used in pam */
 gboolean
-gdm_verify_open_session (GdmDisplay *d)
+gdm_verify_setup_env (GdmDisplay *d)
 {
 	return TRUE;
 }
